<?php

namespace Drupal\Tests\vmware\Functional;

use Drupal\Tests\cloud\Functional\Utils;
use Drupal\Tests\cloud\Traits\CloudConfigTestEntityTrait;

/**
 * Tests Vmware VM.
 *
 * @group Vmware
 */
class VmwareVmTest extends VmwareTestBase {

  use CloudConfigTestEntityTrait;

  public const VMWARE_VM_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions(): array {
    return [
      'list vmware vm',
      'add vmware vm',
      'view vmware vm',
      'edit vmware vm',
      'delete vmware vm',
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  protected function getMockDataTemplateVars(): array {
    return [
      'host_name1' => Utils::getRandomPrivateIp(),
      'host_name2' => Utils::getRandomPrivateIp(),
      'host1' => $this->random->name(8, TRUE),
      'host2' => $this->random->name(8, TRUE),
      'folder_name1' => 'Folder-' . $this->random->name(8, TRUE),
      'folder_name2' => 'Folder-' . $this->random->name(8, TRUE),
      'folder1' => $this->random->name(8, TRUE),
      'folder2' => $this->random->name(8, TRUE),
      'datastore_name1' => 'Datastore-' . $this->random->name(8, TRUE),
      'datastore_name2' => 'Datastore-' . $this->random->name(8, TRUE),
      'datastore1' => $this->random->name(8, TRUE),
      'datastore2' => $this->random->name(8, TRUE),
      'free_space1' => rand(1024 * 1024, 1024 * 1024 * 100),
      'free_space2' => rand(1024 * 1024, 1024 * 1024 * 100),
      'capacity1' => rand(1024 * 1024, 1024 * 1024 * 100),
      'capacity2' => rand(1024 * 1024, 1024 * 1024 * 100),
    ];
  }

  /**
   * Tests CRUD for VM.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testVm(): void {
    $cloud_context = $this->cloudContext;

    // List VM for VMware.
    $this->drupalGet("/clouds/vmware/$cloud_context/vm");
    $this->assertNoErrorMessage();

    // Add a new VM.
    $add = $this->createVmTestFormData(self::VMWARE_VM_REPEAT_COUNT);
    for ($i = 0; $i < self::VMWARE_VM_REPEAT_COUNT; $i++) {
      $this->addVmMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/vmware/$cloud_context/vm/add",
        $add[$i],
        $this->t('Save')
      );
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'VM', '%label' => $add[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been created.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/vmware/$cloud_context/vm");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    for ($i = 0, $num = 1; $i < self::VMWARE_VM_REPEAT_COUNT; $i++, $num++) {
      // Make sure the all VM listing exists.
      $this->drupalGet("/clouds/vmware/$cloud_context/vm");
      $this->assertNoErrorMessage();

      for ($j = 0; $j < $num; $j++) {
        $this->assertSession()->pageTextContains($add[$j]['name']);
      }
    }

    // Delete VM.
    for ($i = 0, $num = 1; $i < self::VMWARE_VM_REPEAT_COUNT; $i++, $num++) {

      $this->deleteVmMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/vmware/$cloud_context/vm/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'VM', '@label' => $add[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type @label has been deleted.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/vmware/$cloud_context/vm");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting VMs with bulk operation.
   *
   * @throws \Exception
   */
  public function testVmBulk(): void {

    for ($i = 0; $i < self::VMWARE_VM_REPEAT_COUNT; $i++) {
      // Create Vms.
      $vms = $this->createVmsRandomTestFormData();
      $entities = [];
      foreach ($vms ?: [] as $vm) {
        $entities[] = $this->createVmTestEntity($vm);
      }
      $this->deleteVmMockData($vms[0]);
      $this->runTestEntityBulk('vm', $entities);
    }
  }

}
